<?php


class nc_payment_system_rbs extends nc_payment_system
{

    const RBS_PROD_URL = 'https://ecom.alfabank.by/payment/rest/';
    const RBS_TEST_URL = 'https://web.rbsuat.com/ab_by/rest/';

    public $version = '2.5.0';
    protected $automatic = TRUE;

    // принимаемые валюты
    protected $accepted_currencies = array('RUB', 'RUR', 'BYN');

    // параметры платежной системы
    protected $settings = array(
        'userName' => null,
        'password' => null,
        'test_mode' => 'yes',
        'two-stage' => 'no',
        'successUrl' => null,
        'failUrl' => null,
        'send_order' => 'no',
        'taxSystem' => null,
        'ffdVersion' => 'v105',
        'paymentMethod' => 1,
        'paymentObject' => 4,
        'write_log' => 'no',
    );

    // передаваемые параметры
    protected $request_parameters = array();

    // получаемые параметры
    protected $callback_response = array(
        'formUrl' => null,
        'orderNumber' => null
    );

    static public $rbs_vats = [
        'none' => 0,
        '0' => 1,
        '10' => 2,
        '18' => 3,
        '20' => 6
    ];


    public function execute_payment_request(nc_payment_invoice $invoice)
    {
        $protocol = isset($_SERVER['HTTPS']) ? 'https://' : 'http://';
        $return_url = $protocol . $_SERVER['SERVER_NAME'] . "/netcat/modules/payment/callback.php?paySystem=nc_payment_system_rbs&type=result";

        $amount = $invoice->get_amount("%0.2F");
        $order_id = $invoice->get("order_id");
        $invoice_id = $invoice->get_id();

        $data = array(
            'userName' => $this->settings['userName'],
            'password' => $this->settings['password'],
            'orderNumber' => $invoice_id . "_" . time(),
            'amount' => $amount * 100,
            'returnUrl' => $return_url,
            'jsonParams' => json_encode(
                [
                    'CMS:' => 'NetCat 5.x',
                    'Module-Version: ' => $this->version
                ]
            ),
        );

        //collect order data
        if ($this->settings['send_order'] == 'yes') {

            $data['taxSystem'] = empty($this->settings['taxSystem']) ? 0 : $this->settings['taxSystem'];

            $items = array();
            $itemsCnt = 1;

            $netshop = nc_netshop::get_instance();
            $order = $netshop->load_order($order_id);
            $collection = $order->get_items();


            $sellReceipt = $invoice->get_sell_receipt();
            foreach ($sellReceipt->get_items() as $i) {
                if ($i['source_item_id']) {

                    $item = array();
                    $product_price = round((float)$i->get('item_price') * 100);

                    $item['positionId'] = $itemsCnt++;
                    $item['name'] = mb_substr($i['name'],0,64);
                    $item['quantity'] = array(
                        'value' => $i->get('qty'),
                        'measure' => 'шт'
                    );
                    $item['itemAmount'] = round($i->get('qty') * (float)$i->get('item_price') * 100);
                    $item['itemCode'] = $i->get('id');
                    $item['tax'] = array(
                        'taxType' => nc_array_value(self::$rbs_vats, $i->get('vat_rate'), self::$rbs_vats['none']),
                    );
                    $item['itemPrice'] = $product_price;

                    // FFD 1.05 added
                    if ($this->settings['ffdVersion'] == "v105") {
                        $attributes = array();
                        $attributes[] = array(
                            "name" => "paymentMethod",
                            "value" => $this->settings['paymentMethod']
                        );
                        $attributes[] = array(
                            "name" => "paymentObject",
                            "value" => $this->settings['paymentObject']
                        );

                        $item['itemAttributes']['attributes'] = $attributes;
                    }
                    $items[] = $item;
                }
            }


            // DELIVERY POSITION
            if (isset($order['DeliveryCost']) && $order['DeliveryCost']) {
                $price = round((float)$order['DeliveryCost'] * 100);
                $itemShipment['positionId'] = $itemsCnt;
                $itemShipment['name'] = mb_substr($order->get_delivery_method()['name'], 0, 64);
                $itemShipment['quantity'] = array(
                    'value' => 1,
                    'measure' => 'шт'
                );
                $itemShipment['itemAmount'] = $price;
                $itemShipment['itemCode'] = 'Delivery';
                $itemShipment['tax'] = array(
                    'taxType' => 0 //todo
                );

                // FFD 1.05 added
                if ($this->settings['ffdVersion'] == "v105") {
                    $attributes = array();
                    $attributes[] = array(
                        "name" => "paymentMethod",
                        "value" => 4
                    );
                    $attributes[] = array(
                        "name" => "paymentObject",
                        "value" => 4
                    );
                    $itemShipment['itemAttributes']['attributes'] = $attributes;
                }
                $items[] = $itemShipment;
            }

            $invoice_data = $invoice->to_array();

            $order_bundle = array(
                'orderCreationDate' => date("c"),
                'customerDetails' => array(
                    'email' => $invoice_data['customer_email'],
                    'phone' => preg_replace("/[^0-9]/", "", nc_normalize_phone_number($invoice_data['customer_phone'])),
                ),
                'cartItems' => array('items' => $items)
            );

            $data['orderBundle'] = json_encode($order_bundle);

        }

        $action_adr = ($this->settings['test_mode'] == 'yes') ? self::RBS_TEST_URL : self::RBS_PROD_URL;

        if ($this->settings['two-stage'] == 'yes') {
            $action_adr .= 'registerPreAuth.do';
        } else {
            $action_adr .= 'register.do';
        }


        $response = $this->execute_payment_session($data, $action_adr);

        if ($response['errorCode'] == 0) {
            header("Location: " . $response['formUrl']);
        } else {
            die("RBSPayment[ERROR]: ".$response['errorMessage']);
        }

        exit;
    }

    public function execute_payment_session($data, $url)
    {
        return $this->curl_post_data($data, $url);
    }

    public function curl_post_data($args, $action_adr)
    {

        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => $action_adr,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => http_build_query($args, '', '&'),
            CURLOPT_HTTPHEADER => array('CMS: NetCat 5.9.x', 'Module-Version: ' . $this->version),
            CURLOPT_SSLVERSION => CURL_SSLVERSION_TLSv1_2,

        ));

        $json = curl_exec($curl);

        if ($json == false) {
            die ('Curl error: ' . curl_error($curl) . '<br>');
        }

        //Преобразуем JSON в ассоциативный массив
        $result = json_decode($json, TRUE);
        curl_close($curl);

        if ( $this->settings['write_log'] == 'yes' ) {
            $this->rbs_logger('Request: ' . $action_adr . ': ' . print_r($args, true) . 'Response: ' . $json);
        }

        return $result;
    }

    /**
     * @param nc_payment_invoice $invoice
     */
    public function on_response(nc_payment_invoice $invoice = null)
    {

        $this->on_payment_success($invoice);
        echo 'OK' . $invoice->get_id();
    }

    /**
     *
     */
    public function validate_payment_request_parameters()
    {

    }

    /**
     * @param nc_payment_invoice $invoice
     */
    public function validate_payment_callback_response(nc_payment_invoice $invoice = null)
    {

        $data = array(
            'userName' => $this->settings['userName'],
            'password' => $this->settings['password'],
            'orderId' => $this->get_response_value('orderId'),
        );

        $url = ($this->settings['test_mode'] == 'yes') ? self::RBS_TEST_URL : self::RBS_PROD_URL;
        $response = $this->curl_post_data($data, $url . "getOrderStatusExtended.do");
        $orderStatus = $response['orderStatus'];

        if ($orderStatus != '1' && $orderStatus != '2') {
            $this->add_error($response['errorMessage']);
            header("Location: " . $this->get_setting('failUrl'));
            exit;
        }

        $nc_order_number = explode("_", $response['orderNumber']);
        $invoice = $this->load_invoice($nc_order_number[0]);

        $invoice->set('status', nc_payment_invoice::STATUS_SUCCESS);
        $invoice->save();
        $this->on_payment_success($invoice);

        header("Location: " . $this->get_setting('successUrl'));

    }

    public function load_invoice_on_callback()
    {
        return $this->load_invoice($this->get_response_value('orderId'));
    }

    function rbs_logger($data)
    {
        if ($data) {

            $result = $data;
            if (is_array($data) || is_object($data)) {
                $result = "\n" . print_r($data, true);
            }
            $result .= "\n\n";
            $path = dirname(__FILE__) . '/rbs_payment.log';
            error_log($result, 3, $path);
            return true;
        }
        return false;
    }

}
